@@ -60,7 +60,7 @@ class BaseGopherProtocol:
             self.filenotfound(e[1])
 
     def filenotfound(self, msg):
-        self.wfile.write("0%s\t\terror.host\t1\r\n" % msg)
+        self.wfile.write("3%s\t\terror.host\t1\r\n" % msg)
 
     def renderobjinfo(self, entry):
         """Renders an object's info according to the protocol.  Returns
@@ -71,3 +71,12 @@ class BaseGopherProtocol:
         """Gets the handler for this object's selector."""
         return handlers.HandlerMultiplexer.getHandler(self.selector,
                                                self, self.config)
+
+    def renderdirstart(self, entry):
+        """Renders the start of a directory.  Most protocols will not need
+        this.  Exception might be HTML.  Returns None if not needed."""
+        return None
+
+    def renderdirend(self, entry):
+        """Likewise for the end of a directory."""
+        return None
